/*************************************************
 *	 ___       _             _     
 *	|   |_ _ _| |___ ___   _| |___ 
 *	| | |_'_| . | -_|  _|_| . | -_|
 *	|___|_,_|___|___|___|_|___|___|
 * 
 * FILE: 	lib_clk.c
 * Author: 	declis (0xdec.de)
 ************************************************/

#include <msp430g2452.h>
#include "lib_clk.h"
#include "lib_system.h"
#include "lib_lcd.h"
#include "def_ports.h"
#include "delay.h"		

/****************************************************************************************** 
 * Name: 		calc_time
 * Description: increment seconds and calculate new time
 * Input:		*time (struct the_time, call by pointer)
 * Return:		nothing	
 *****************************************************************************************/ 

void calc_time(struct the_time *time)
{
  	time->ss++;
  	if(time->ss>59)
  	{
  		time->ss=0;
  		time->mm++;
  		if(time->mm>59)
  		{
  			time->mm=0;
  			time->hh++;
  			if(time->hh>23) time->hh=0;
  		}
  	}
}

/****************************************************************************************** 
 * Name: 		calc_date
 * Description: increment days and calculate new date
 * Input:		*date (struct the_time), days_month
 * Return:		nothing	
 *****************************************************************************************/ 
 
void calc_date(struct the_date *date, const char *days_month)
{
	date->day++;
	// if the day overflows the current month length, it's a new month
	if(date->day>days_month[date->month-1])
	{	
		if(reset_day(date))
			date->month++;
		if(date->month>12)
		{
			date->month=1;
			date->year++;
			date->leapyear=is_leapyear(date->year);
		}
	}
}

/****************************************************************************************** 
 * Name: 		reset_day
 * Description: - if the current year is a leapyear and the current month is Feb,
 *				  the month length is 29
 *				- if the current year is a leapyear and the current month is Feb 
 *				  and the day overflows 29 -> new month, reset day
 * Input:		*date (struct the_time)
 * Return:		nothing	
 *****************************************************************************************/ 
 
unsigned char reset_day(struct the_date *date)
{
	if(date->leapyear&&date->month==2&&date->day==29) return 0;
	return date->day=1;
}

/****************************************************************************************** 
 * Name: 		get_dayofweek
 * Description: Zeller's congruence, see wikipedia
 * Input:		date (struct the_time)
 * Return:		0=Monday...6=Sunday
 *****************************************************************************************/ 

unsigned char get_dayofweek(struct the_date date)
{
	if(date.month<3)
	{
		date.month+=12;
		date.year--;
	}
	return((13*date.month+3)/5+date.day+date.year+date.year/4-date.year/100+date.year/400)%7;
}

/****************************************************************************************** 
 * Name: 		is_leapyear
 * Description: leapyear check
 * Input:		year
 * Return:		0=no leapyear, 1=leapyear true
 *****************************************************************************************/ 

unsigned char is_leapyear(unsigned int year)
{
	if(!(year%4)&&(year%100)||!(year%400)) return 1;
	return 0;
}

void set_lcd_time(struct the_time time)
{
	char_to_num_out(hh_x,hh_y,time.hh);
	write_char(hh_x+2,hh_y,':');
	char_to_num_out(mm_x,mm_y,time.mm);
	write_char(mm_x+2,mm_y,':');
	char_to_num_out(ss_x,ss_y,time.ss);
}

void set_lcd_date(struct the_date date, const char day_name[][str_length])
{
	char_to_num_out(day_x,day_y,date.day);				// print day
	write_char(day_x+2,day_y,'.');
	char_to_num_out(month_x,month_y,date.month);		// print month
	write_char(month_x+2,month_y,'.');
	char_to_num_out(year_x,year_y,date.year/100);		// print century 
	char_to_num_out(year_x+2,year_y,date.year%100);		// print year
	write_string(day_name_x,day_name_y,day_name[get_dayofweek(date)]);	// print day of week
}

void set_numitron_time(struct the_time time)
{
	// in setup mode, tube seconds are always enabled
	if(time.tube_ss||setup_mode)						// seconds enabled
		send_number(time.ss);
	else
		send_shift_register(numitron_off);				// seconds disabled
	send_number(time.mm);
	send_number(time.hh);
}

void one_tube_off(unsigned char tube_num,struct the_time time)
{
	if(tube_num==1)
	{
		send_number(time.ss);
		send_number(time.mm);
		send_shift_register(numitron_off);
	}
	else if(tube_num==2)
	{
		send_number(time.ss);
		send_shift_register(numitron_off);
		send_number(time.hh);
	}
	else
	{
		send_shift_register(numitron_off);
		send_number(time.mm);
		send_number(time.hh);
	}
}

#pragma INTERRUPT (ISR_TimerA);
#pragma vector=TIMER0_A0_VECTOR;
void ISR_TimerA(void)
{
	LPM3_EXIT;
}

