/*************************************************
 *	 ___       _             _     
 *	|   |_ _ _| |___ ___   _| |___ 
 *	| | |_'_| . | -_|  _|_| . | -_|
 *	|___|_,_|___|___|___|_|___|___|
 * 
 * FILE: 	main.c
 * Author: 	declis (0xdec.de)
 ************************************************/

#include <msp430g2452.h>
#include "def_ports.h"
#include "lib_clk.h"
#include "lib_system.h"
#include "lib_lcd.h"

const char days_month[]={31,28,31,30,31,30,31,31,30,31,30,31};
const char day_name[][str_length]={"MO","DI","MI","DO","FR","SA","SO"};
const char centigrade[]={0xF2,'C',0x00};
const char setup_tube_sec[]={"s="};

unsigned char rgb=0,setup_mode=0,setup_blink=0;
struct the_date date;
struct the_time time;

void main(void)
{	
	WDTCTL=WDTPW+WDTHOLD;				// stop WDT
	init_ports();						// init. Ports
	init_DCO();							// init. DCO (1MHz)
	init_ACLK();						// init. ACLK (32kHz Crystal)
	init_ADC();							// init. ADC (Temperaturesensor)
	init_USI();							// init. USI (SPI-Mode for LCD)
	init_LCD();							// init. LCD (DOGM162)
	
	date.day=1;
	date.month=1;
	date.year=2013;
	date.leapyear=0;
	time.hh=23;
	time.mm=59;
	time.ss=55;
	time.tube_ss=1;						// tube seconds enabled by default
	
	set_lcd_date(date,day_name);		// write date to display
	write_string(c_x,c_y,centigrade);	// wirte "C" to display
	init_TimerA();						// init. TimerA (Interrupt every 1 sec)
	
  	while(1)
  	{	
  		// NORMAL MODE IS ACTIVE------------------------------------------------------------
  		if(!setup_mode)
  		{
  			// Vcc is on, MSP430 not running on battery
  			if(P1IN&vcc_check)
  			{
  				set_numitron_time(time);
  				set_lcd_time(time);
  				set_lcd_date(date,day_name);
  				// read and print temperature every 2 seconds
  				if(!(time.ss%2))
					char_to_num_out(temp_c_x,temp_c_y,get_temp());
  			}
  			// calculate new time
  			calc_time(&time);
  			
  			// if new day started, get new date (00:00:00)
  			if(!time.hh&&!time.mm&&!time.ss)
				calc_date(&date,days_month);
  		}
  		// SETUP MODE IS ACTIVE-------------------------------------------------------------
  		else
  		{	
  			// the selected digit will blink every 500ms
  			if(setup_blink)
  			{
  				if(setup_mode==1)
  				{
  					one_tube_off(1,time);				// hour tubes off
  					clean_area(hh_x,hh_y,2);			// LCD hh off
  				}
  				else if(setup_mode==2)
  				{
  					one_tube_off(2,time);				// minute tubes off
  					clean_area(mm_x,mm_y,2);			// LCD mm off
  				}
  				else if(setup_mode==3)
  					clean_area(day_x,day_y,2);			// day off (LCD)
  				else if(setup_mode==4)
  					clean_area(month_x,month_y,2);		// month off (LCD)
  				else if(setup_mode==5)
  					clean_area(year_x,year_y,4);		// year off (LCD)
  				else if(setup_mode==6)
  					clean_area(tube_ss_x,tube_ss_y,2);	// tube seconds option off (LCD)
  			}
  			else
  			{
  				set_numitron_time(time);
  				set_lcd_time(time);
  				set_lcd_date(date,day_name);
  				char_to_num_out(tube_ss_x,tube_ss_y,time.tube_ss);
  			}
  			
  			setup_blink^=1;							// toggle setup_blink
  			
  			// EXTI SETUP MODE
  			if(setup_mode>setup_end)				
  			{
  				TAR=0;							
				TACCR0=cycles_1000;	
  			  	setup_mode=0;							
  			  	setup_blink=0;					
				clean_area(tube_ss_x-sizeof(setup_tube_sec)+1,tube_ss_y,sizeof(setup_tube_sec)+1);	// delete tube seconds option 
  			}
  		}
  		
  		// after the main routine is done, enter LPM3, wait for next Timer-Inerrupt 
		LPM3;
  	}	
}

#pragma INTERRUPT (ISR_Port1);
#pragma vector=PORT1_VECTOR;
void ISR_Port1(void)
{
	// VCC-CHECK-HANDLING-------------------------------------------------------------------
	if(P1IFG&vcc_check)
	{	
		//- if Vcc is down, an interrupt will be triggered
		//  on a Hi/Lo edge => battery mode is active
		//- if Vcc is back on, an interrupt will be triggered again
		//  on a Lo/Hi edge 
		P1IES^=vcc_check;
		
		// bat. mode active, minimize power consumption as much as possible
		if(!(P1IN&vcc_check))	
			P2OUT&=~(data+clock+strobe+red+green+blue);	
		// bat mode deactivated, LCD (not powered by bat.) needs a new init.	
		else
		{
			init_LCD();
			set_lcd_date(date,day_name);
			write_string(c_x,c_y,centigrade);
		}
	}
	// KEY-SETUP-MODE-HANDLING--------------------------------------------------------------
	if(P1IFG&s_mode)
	{
		// start setup mode
		if(!setup_mode) 
		{
			setup_mode=1;
			TAR=0;
			TACCR0=cycles_500;
			time.ss=0;
			write_string(tube_ss_x-sizeof(setup_tube_sec)+1,tube_ss_y,setup_tube_sec);
			write_char(tube_ss_x,tube_ss_y,time.tube_ss);
			// maybe the calc_time function interrupted, safety first
			if(time.hh>59) time.hh=0;
			if(time.mm>59) time.mm=0;
		}
		// increment setup_mode (switch through digits)
		else 
		{
			setup_mode++;
  			// SETUP MODE WILL EXIT HERE, Timer-Reset in main()
  			if(setup_mode>setup_end) 
  			{
  				setup_blink=0;
				LPM3_EXIT;
  			}
		}
	}
	// KEY-UP-HANDLING----------------------------------------------------------------------
	if(P1IFG&s_up)
	{	
		// NORMAL MODE, set RGB LEDs (switch through colors)
		if(!setup_mode)
		{
			rgb++;
			rgb=set_rgb(rgb);
		}
		// SETUP MODE starting here
		// count hours up
		else if(setup_mode==1)
		{
			time.hh++;
			if(time.hh>23) time.hh=0;
		}
		// count minutes up
		else if(setup_mode==2)
		{
			time.mm++;
			if(time.mm>59) time.mm=0;
		}
		// count days up
		else if(setup_mode==3)
		{
			date.day++;
			if(date.day>days_month[date.month-1])
				reset_day(&date);
		}
		// count months up
		else if(setup_mode==4)
		{
			date.month++;
			if(date.month>12) 
				date.month=1;
			if(date.day>days_month[date.month-1])
			{
				date.day=days_month[date.month-1];
				if(date.leapyear&&date.month==2)
					date.day++;		
			}
		}
		// count years up
		else if(setup_mode==5)
		{
			date.year++;
			if(date.year>cal_end)
				date.year=cal_start;
			date.leapyear=is_leapyear(date.year);
			if(!date.leapyear&&date.month==2&&date.day==29)
				date.day--;
		}
		// enable or disable tube seconds
		else time.tube_ss^=1;	
	}
	// KEY-DOWN-HANDLING-------------------------------------------------------------------
	if(P1IFG&s_down)
	{	
		// NORMAL MODE, turn all LEDs off
		if(!setup_mode) 
			P2OUT&=~(red+green+blue);
		// SETUP MODE starting here
		// count hours down
		else if(setup_mode==1)
		{
			time.hh--;
			if(time.hh==255) time.hh=23;
		}
		// count minutes down
		else if(setup_mode==2)
		{
			time.mm--;
			if(time.mm==255) time.mm=59;
		}
		// count days down
		else if(setup_mode==3)
		{
			date.day--;
			if(!date.day)
			{ 
				date.day=days_month[date.month-1];
				if(date.leapyear&&date.month==2)
					date.day=29;
			}
		}
		// count months down
		else if(setup_mode==4)
		{
			date.month--;
			if(!date.month) 
				date.month=12;
			if(date.day>days_month[date.month-1])
			{
				date.day=days_month[date.month-1];
				if(date.leapyear&&date.month==2)
					date.day++;
			}
		}
		// count years down
		else if(setup_mode==5)
		{
			date.year--;
			if(date.year<cal_start)
				date.year=cal_end;
			date.leapyear=is_leapyear(date.year);
			if(!date.leapyear&&date.month==2&&date.day==29)
				date.day--;
		}
		// enable or disable tube seconds
		else time.tube_ss^=1;
	}
	
	// ISR is done, clean P1IFG
	P1IFG&=~(s_mode+s_up+s_down+vcc_check);
}
